@@ -20,9 +20,9 @@
 # This metadata block gets parsed by setup.py - use single quotes only
 __tagline__ = 'Media aggregator and podcast client'
 __author__ = 'Thomas Perl <thp@gpodder.org>'
-__version__ = '3.10.11'
-__date__ = '2019-09-29'
-__copyright__ = '© 2005-2019 The gPodder Team'
+__version__ = '3.10.19'
+__date__ = '2021-04-15'
+__copyright__ = '© 2005-2021 The gPodder Team'
 __license__ = 'GNU General Public License, version 3 or later'
 __url__ = 'http://gpodder.org/'
 
@@ -82,8 +82,14 @@ except ImportError:
 del sqlite3
 
 
+# Is gpodder running in verbose mode?
+verbose = False
+# Is gpodder running in quiet mode?
+quiet = False
+
+
 # The User-Agent string for downloads
-user_agent = 'gPodder/%s (+%s) %s/%s' % (__version__, __url__, platform.system(), platform.release())
+user_agent = 'gPodder/%s (+%s) %s' % (__version__, __url__, platform.system())
 
 
 # Are we running in GUI or console mode?
@@ -137,7 +143,7 @@ del locale_dir
 SOCKET_TIMEOUT = 60
 socket.setdefaulttimeout(SOCKET_TIMEOUT)
 del socket
-del SOCKET_TIMEOUT
+SOCKET_TIMEOUT
 
 # Variables reserved for GUI-specific use (will be set accordingly)
 ui_folders = []
@@ -179,7 +185,8 @@ def fixup_home(old_home):
             old_home = new_home  # force to config directory
             print("D: windows-portable build; forcing home to config directory %s" % new_home, file=sys.stderr)
         else:  # ui.win32, not portable build
-            from gpodder.utilwin32ctypes import get_documents_folder, get_reg_current_user_string_value
+            from gpodder.utilwin32ctypes import (
+                get_documents_folder, get_reg_current_user_string_value)
             try:
                 # from old launcher, see
                 # https://github.com/gpodder/gpodder/blob/old/gtk2/tools/win32-launcher/folderselector.c
