@@ -162,6 +162,7 @@ defaults = {
                 'descriptions': True,
                 'view_mode': 1,
                 'columns': int('110', 2),  # bitfield of visible columns
+                'always_show_new': True,
             },
 
             'download_list': {
@@ -218,8 +219,6 @@ defaults = {
 
 # The sooner this goes away, the better
 gPodderSettings_LegacySupport = {
-    'player': 'player.audio',
-    'videoplayer': 'player.video',
     'limit_rate': 'limit.bandwidth.enabled',
     'limit_rate_value': 'limit.bandwidth.kbps',
     'max_downloads_enabled': 'limit.downloads.enabled',
