@@ -79,7 +79,11 @@ class gPodderChannel(BuilderWidget):
             self.btn_website.hide()
 
         b = Gtk.TextBuffer()
-        b.set_text(self.channel.description)
+        if self.channel._update_error:
+            err = '\n\nERROR: {}'.format(self.channel._update_error)
+        else:
+            err = ''
+        b.set_text(util.remove_html_tags(self.channel.description) + err)
         self.channel_description.set_buffer(b)
 
         # Add Drag and Drop Support
@@ -94,6 +98,13 @@ class gPodderChannel(BuilderWidget):
         self.imgCoverEventBox.connect('button-press-event',
                 self.on_cover_popup_menu)
 
+        gpodder.user_extensions.on_ui_object_available('channel-gtk', self)
+
+        result = gpodder.user_extensions.on_channel_settings(self.channel)
+        if result:
+            for label, callback in result:
+                self.notebookChannelEditor.append_page(callback(self.channel), Gtk.Label(label))
+
     def on_button_add_section_clicked(self, widget):
         text = self.show_text_edit_dialog(_('Add section'), _('New section:'),
             affirmative_text=Gtk.STOCK_ADD)
