@@ -112,37 +112,50 @@ class YouTubeVideoFormatListModel(Gtk.ListStore):
     def __init__(self, config):
         Gtk.ListStore.__init__(self, str, int)
         self._config = config
-        self.custom_fmt_ids = self._config.youtube.preferred_fmt_ids
 
         if self._config.youtube.preferred_fmt_ids:
             caption = _('Custom (%(format_ids)s)') % {
-                'format_ids': ', '.join(str(x) for x in self.custom_fmt_ids),
+                'format_ids': ', '.join(str(x) for x in self._config.youtube.preferred_fmt_ids),
             }
-            self.append((caption, -1))
+            self.append((caption, 0))
 
         for id, (fmt_id, path, description) in youtube.formats:
             self.append((description, id))
 
     def get_index(self):
-        if self._config.youtube.preferred_fmt_ids:
-            return 0
-
         for index, row in enumerate(self):
             if self._config.youtube.preferred_fmt_id == row[self.C_ID]:
                 return index
         return 0
 
     def set_index(self, index):
-        value = self[index][self.C_ID]
-        if value > 0:
-            self._config.youtube.preferred_fmt_id = value
-            # If we set a value, we need to unset the custom one, so that
-            # the single value (preferred_fmt_id) gets used instead
-            self._config.youtube.preferred_fmt_ids = []
-        else:
-            # If the user selects the -1 value, it's our custom one, and
-            # we need to restore the value for preferred_fmt_ids
-            self._config.youtube.preferred_fmt_ids = self.custom_fmt_ids
+        self._config.youtube.preferred_fmt_id = self[index][self.C_ID]
+
+
+class YouTubeVideoHLSFormatListModel(Gtk.ListStore):
+    C_CAPTION, C_ID = list(range(2))
+
+    def __init__(self, config):
+        Gtk.ListStore.__init__(self, str, int)
+        self._config = config
+
+        if self._config.youtube.preferred_hls_fmt_ids:
+            caption = _('Custom (%(format_ids)s)') % {
+                'format_ids': ', '.join(str(x) for x in self._config.youtube.preferred_hls_fmt_ids),
+            }
+            self.append((caption, 0))
+
+        for id, (fmt_id, path, description) in youtube.hls_formats:
+            self.append((description, id))
+
+    def get_index(self):
+        for index, row in enumerate(self):
+            if self._config.youtube.preferred_hls_fmt_id == row[self.C_ID]:
+                return index
+        return 0
+
+    def set_index(self, index):
+        self._config.youtube.preferred_hls_fmt_id = self[index][self.C_ID]
 
 
 class VimeoVideoFormatListModel(Gtk.ListStore):
@@ -183,12 +196,12 @@ class gPodderPreferences(BuilderWidget):
 
         self.audio_player_model = self.user_apps_reader.get_model('audio')
         self.combo_audio_player_app.set_model(self.audio_player_model)
-        index = self.audio_player_model.get_index(self._config.player)
+        index = self.audio_player_model.get_index(self._config.player.audio)
         self.combo_audio_player_app.set_active(index)
 
         self.video_player_model = self.user_apps_reader.get_model('video')
         self.combo_video_player_app.set_model(self.video_player_model)
-        index = self.video_player_model.get_index(self._config.videoplayer)
+        index = self.video_player_model.get_index(self._config.player.video)
         self.combo_video_player_app.set_active(index)
 
         self.preferred_youtube_format_model = YouTubeVideoFormatListModel(self._config)
@@ -198,6 +211,13 @@ class gPodderPreferences(BuilderWidget):
         self.combobox_preferred_youtube_format.add_attribute(cellrenderer, 'text', self.preferred_youtube_format_model.C_CAPTION)
         self.combobox_preferred_youtube_format.set_active(self.preferred_youtube_format_model.get_index())
 
+        self.preferred_youtube_hls_format_model = YouTubeVideoHLSFormatListModel(self._config)
+        self.combobox_preferred_youtube_hls_format.set_model(self.preferred_youtube_hls_format_model)
+        cellrenderer = Gtk.CellRendererText()
+        self.combobox_preferred_youtube_hls_format.pack_start(cellrenderer, True)
+        self.combobox_preferred_youtube_hls_format.add_attribute(cellrenderer, 'text', self.preferred_youtube_hls_format_model.C_CAPTION)
+        self.combobox_preferred_youtube_hls_format.set_active(self.preferred_youtube_hls_format_model.get_index())
+
         self.preferred_vimeo_format_model = VimeoVideoFormatListModel(self._config)
         self.combobox_preferred_vimeo_format.set_model(self.preferred_vimeo_format_model)
         cellrenderer = Gtk.CellRendererText()
@@ -280,14 +300,21 @@ class gPodderPreferences(BuilderWidget):
         self.entry_username.set_text(self._config.mygpo.username)
         self.entry_password.set_text(self._config.mygpo.password)
         self.entry_caption.set_text(self._config.mygpo.device.caption)
-        self.entry_youtube_api_key.set_text(self._config.youtube.api_key_v3)
 
         # Disable mygpo sync while the dialog is open
         self._config.mygpo.enabled = False
 
         # Configure the extensions manager GUI
         self.set_extension_preferences()
-        self.main_window.show()
+
+        self._config.connect_gtk_window(self.main_window, 'preferences', True)
+
+        gpodder.user_extensions.on_ui_object_available('preferences-gtk', self)
+
+        result = gpodder.user_extensions.on_preferences()
+        if result:
+            for label, callback in result:
+                self.notebook.append_page(callback(), Gtk.Label(label))
 
     def _extensions_select_function(self, selection, model, path, path_currently_selected):
         return model.get_value(model.get_iter(path), self.C_SHOW_TOGGLE)
@@ -457,16 +484,20 @@ class gPodderPreferences(BuilderWidget):
 
     def on_combo_audio_player_app_changed(self, widget):
         index = self.combo_audio_player_app.get_active()
-        self._config.player = self.audio_player_model.get_command(index)
+        self._config.player.audio = self.audio_player_model.get_command(index)
 
     def on_combo_video_player_app_changed(self, widget):
         index = self.combo_video_player_app.get_active()
-        self._config.videoplayer = self.video_player_model.get_command(index)
+        self._config.player.video = self.video_player_model.get_command(index)
 
     def on_combobox_preferred_youtube_format_changed(self, widget):
         index = self.combobox_preferred_youtube_format.get_active()
         self.preferred_youtube_format_model.set_index(index)
 
+    def on_combobox_preferred_youtube_hls_format_changed(self, widget):
+        index = self.combobox_preferred_youtube_hls_format.get_active()
+        self.preferred_youtube_hls_format_model.set_index(index)
+
     def on_combobox_preferred_vimeo_format_changed(self, widget):
         index = self.combobox_preferred_vimeo_format.get_active()
         self.preferred_vimeo_format_model.set_index(index)
@@ -474,21 +505,21 @@ class gPodderPreferences(BuilderWidget):
     def on_button_audio_player_clicked(self, widget):
         result = self.show_text_edit_dialog(_('Configure audio player'),
                                             _('Command:'),
-                                            self._config.player)
+                                            self._config.player.audio)
 
         if result:
-            self._config.player = result
-            index = self.audio_player_model.get_index(self._config.player)
+            self._config.player.audio = result
+            index = self.audio_player_model.get_index(self._config.player.audio)
             self.combo_audio_player_app.set_active(index)
 
     def on_button_video_player_clicked(self, widget):
         result = self.show_text_edit_dialog(_('Configure video player'),
                                             _('Command:'),
-                                            self._config.videoplayer)
+                                            self._config.player.video)
 
         if result:
-            self._config.videoplayer = result
-            index = self.video_player_model.get_index(self._config.videoplayer)
+            self._config.player.video = result
+            index = self.video_player_model.get_index(self._config.player.video)
             self.combo_video_player_app.set_active(index)
 
     def format_update_interval_value(self, scale, value):
@@ -542,12 +573,6 @@ class gPodderPreferences(BuilderWidget):
         # Only update indirectly (see on_dialog_destroy)
         self._enable_mygpo = widget.get_active()
 
-    def on_youtube_api_key_changed(self, widget):
-        self._config.youtube.api_key_v3 = widget.get_text()
-
-    def on_button_youtube_api_key_clicked(self, widget):
-        util.open_website('http://wiki.gpodder.org/wiki/Youtube')
-
     def on_server_changed(self, widget):
         self._config.mygpo.server = widget.get_text()
 
