@@ -35,10 +35,10 @@ import shutil
 import string
 import time
 
-import gpodder
 import podcastparser
-from gpodder import (coverart, escapist_videos, feedcore, registry, schema,
-                     util, vimeo, youtube)
+
+import gpodder
+from gpodder import coverart, feedcore, registry, schema, util, vimeo, youtube
 
 logger = logging.getLogger(__name__)
 
@@ -148,12 +148,23 @@ class PodcastParserFeed(Feed):
         if self.max_episodes > 0 and len(entries) > self.max_episodes:
             entries = entries[:self.max_episodes]
 
+        num_duplicate_guids = 0
+
         # Search all entries for new episodes
         for entry in entries:
             episode = channel.EpisodeClass.from_podcastparser_entry(entry, channel)
             if episode is None:
                 continue
 
+            # Discard episode when its GUID collides with a newer episode
+            if episode.guid in seen_guids:
+                num_duplicate_guids += 1
+                channel._update_error = ('Discarded {} episode(s) with non-unique GUID, contact the podcast publisher to fix this issue.'
+                        .format(num_duplicate_guids))
+                logger.warn('Discarded episode with non-unique GUID, contact the podcast publisher to fix this issue. [%s] [%s]',
+                        channel.title, episode.title)
+                continue
+
             seen_guids.add(episode.guid)
             # Detect (and update) existing episode based on GUIDs
             existing_episode = existing_guids.get(episode.guid, None)
@@ -171,37 +182,40 @@ class PodcastParserFeed(Feed):
             url = self.feed['paged_feed_next']
             logger.debug("get_next_page: feed has next %s", url)
             url = channel.authenticate_url(url)
-            res = self.fetcher.fetch(url, max_episodes=max_episodes)
-            if res.status == feedcore.UPDATED_FEED:
-                res.feed = PodcastParserFeed(res.feed, self.fetcher, max_episodes)
-            return res
+            return self.fetcher.fetch(url, autodiscovery=False, max_episodes=max_episodes)
         return None
 
 
 class gPodderFetcher(feedcore.Fetcher):
     """
-    This class extends the feedcore Fetcher with the gPodder User-Agent and the
-    Proxy handler based on the current settings in gPodder.
+    This class implements fetching a channel from custom feed handlers
+    or the default using podcastparser
     """
     def fetch_channel(self, channel, max_episodes):
         custom_feed = registry.feed_handler.resolve(channel, None, max_episodes)
         if custom_feed is not None:
             return custom_feed
+        # TODO: revisit authenticate_url: pass auth as kwarg
         # If we have a username or password, rebuild the url with them included
         # Note: using a HTTPBasicAuthHandler would be pain because we need to
         # know the realm. It can be done, but I think this method works, too
         url = channel.authenticate_url(channel.url)
-        res = self.fetch(url, channel.http_etag, channel.http_last_modified, max_episodes)
-        if res.status == feedcore.UPDATED_FEED:
-            res.feed = PodcastParserFeed(res.feed, self, max_episodes)
-        return res
+        return self.fetch(url, channel.http_etag, channel.http_last_modified, max_episodes=max_episodes)
 
     def _resolve_url(self, url):
         url = youtube.get_real_channel_url(url)
         url = vimeo.get_real_channel_url(url)
-        url = escapist_videos.get_real_channel_url(url)
         return url
 
+    def parse_feed(self, url, data_stream, headers, status, max_episodes=0, **kwargs):
+        try:
+            feed = podcastparser.parse(url, data_stream)
+            feed['url'] = url
+            feed['headers'] = headers
+            return feedcore.Result(status, PodcastParserFeed(feed, self, max_episodes))
+        except ValueError as e:
+            raise feedcore.InvalidFeed('Could not parse feed: {url}: {msg}'.format(url=url, msg=e))
+
 
 # Our podcast model:
 #
@@ -242,10 +256,11 @@ class PodcastEpisode(PodcastModelObject):
     # In theory, Linux can have 255 bytes (not characters!) in a filename, but
     # filesystems like eCryptFS store metadata in the filename, making the
     # effective number of characters less than that. eCryptFS recommends
-    # 140 chars, we use 120 here (140 - len(extension) - len(".partial")).
+    # 140 chars, we use 120 here (140 - len(extension) - len(".partial.webm"))
+    # (youtube-dl appends an extension after .partial, ".webm" is the longest).
     # References: gPodder bug 1898, http://unix.stackexchange.com/a/32834
     MAX_FILENAME_LENGTH = 120  # without extension
-    MAX_FILENAME_WITH_EXT_LENGTH = 140 - len(".partial")  # with extension
+    MAX_FILENAME_WITH_EXT_LENGTH = 140 - len(".partial.webm")  # with extension
 
     __slots__ = schema.EpisodeColumns
 
@@ -285,8 +300,8 @@ class PodcastEpisode(PodcastModelObject):
                 continue
 
             # If we have audio or video available later on, skip
-            # 'application/octet-stream' data types (fixes Linux Outlaws)
-            if episode.mime_type == 'application/octet-stream' and (audio_available or video_available):
+            # all 'application/*' data types (fixes Linux Outlaws and peertube feeds)
+            if episode.mime_type.startswith('application/') and (audio_available or video_available):
                 continue
 
             episode.url = util.normalize_feed_url(enclosure['url'])
@@ -301,7 +316,7 @@ class PodcastEpisode(PodcastModelObject):
         if not episode.url:
             return None
 
-        if any(mod.is_video_link(episode.url) for mod in (youtube, vimeo, escapist_videos)):
+        if any(mod.is_video_link(episode.url) for mod in (youtube, vimeo)):
             return episode
 
         # Check if we can resolve this link to a audio/video file
@@ -385,7 +400,7 @@ class PodcastEpisode(PodcastModelObject):
         # "#001: Title" -> "001: Title"
         if (
                 not self.parent._common_prefix and
-                re.match('^#\d+: ', self.title) and
+                re.match(r'^#\d+: ', self.title) and
                 len(self.title) - 1 > LEFTOVER_MIN):
             return self.title[1:]
 
@@ -463,7 +478,7 @@ class PodcastEpisode(PodcastModelObject):
     def one_line_description(self):
         MAX_LINE_LENGTH = 120
         desc = util.remove_html_tags(self.description or '')
-        desc = re.sub('\s+', ' ', desc).strip()
+        desc = re.sub(r'\s+', ' ', desc).strip()
         if not desc:
             return _('No description available')
         else:
@@ -500,7 +515,7 @@ class PodcastEpisode(PodcastModelObject):
 
         if url is None or not os.path.exists(url):
             # FIXME: may custom downloaders provide the real url ?
-            url = registry.download_url.resolve(config, self.url, self)
+            url = registry.download_url.resolve(config, self.url, self, allow_partial)
         return url
 
     def find_unique_file_name(self, filename, extension):
@@ -574,7 +589,6 @@ class PodcastEpisode(PodcastModelObject):
             # Use title for YouTube, Vimeo and Soundcloud downloads
             if (youtube.is_video_link(self.url) or
                     vimeo.is_video_link(self.url) or
-                    escapist_videos.is_video_link(self.url) or
                     episode_filename == 'stream'):
                 episode_filename = self.title
 
@@ -664,7 +678,7 @@ class PodcastEpisode(PodcastModelObject):
 
     def file_type(self):
         # Assume all YouTube/Vimeo links are video files
-        if youtube.is_video_link(self.url) or vimeo.is_video_link(self.url) or escapist_videos.is_video_link(self.url):
+        if youtube.is_video_link(self.url) or vimeo.is_video_link(self.url):
             return 'video'
 
         return util.file_type_by_extension(self.extension())
@@ -761,7 +775,7 @@ class PodcastEpisode(PodcastModelObject):
 
 
 class PodcastChannel(PodcastModelObject):
-    __slots__ = schema.PodcastColumns + ('_common_prefix',)
+    __slots__ = schema.PodcastColumns + ('_common_prefix', '_update_error',)
 
     UNICODE_TRANSLATE = {ord('ö'): 'o', ord('ä'): 'a', ord('ü'): 'u'}
 
@@ -813,6 +827,8 @@ class PodcastChannel(PodcastModelObject):
             self.children = self.db.load_episodes(self, self.episode_factory)
             self._determine_common_prefix()
 
+        self._update_error = None
+
     @property
     def model(self):
         return self.parent
@@ -933,7 +949,7 @@ class PodcastChannel(PodcastModelObject):
     @classmethod
     def sort_key(cls, podcast):
         key = util.convert_bytes(podcast.title.lower())
-        return re.sub('^the ', '', key).translate(cls.UNICODE_TRANSLATE)
+        return re.sub(r'^the ', '', key).translate(cls.UNICODE_TRANSLATE)
 
     @classmethod
     def load(cls, model, url, create=True, authentication_tokens=None, max_episodes=0):
@@ -991,7 +1007,7 @@ class PodcastChannel(PodcastModelObject):
 
     def _consume_updated_title(self, new_title):
         # Replace multi-space and newlines with single space (Maemo bug 11173)
-        new_title = re.sub('\s+', ' ', new_title).strip()
+        new_title = re.sub(r'\s+', ' ', new_title).strip()
 
         # Only update the podcast-supplied title when we
         # don't yet have a title, or if the title is the
@@ -1135,6 +1151,7 @@ class PodcastChannel(PodcastModelObject):
             if result.status == feedcore.UPDATED_FEED:
                 self._consume_updated_feed(result.feed, max_episodes)
             elif result.status == feedcore.NEW_LOCATION:
+                # FIXME: could return the feed because in autodiscovery it is parsed already
                 url = result.feed
                 logger.info('New feed location: %s => %s', self.url, url)
                 if url in set(x.url for x in self.model.get_podcasts()):
@@ -1198,7 +1215,7 @@ class PodcastChannel(PodcastModelObject):
         return self.section
 
     def _get_content_type(self):
-        if 'youtube.com' in self.url or 'vimeo.com' in self.url or 'escapistmagazine.com' in self.url:
+        if 'youtube.com' in self.url or 'vimeo.com' in self.url:
             return _('Video')
 
         audio, video, other = 0, 0, 0
@@ -1339,7 +1356,7 @@ class Model(object):
 
     def _remove_podcast(self, podcast):
         self.children.remove(podcast)
-        gpodder.user_extensions.on_podcast_delete(self)
+        gpodder.user_extensions.on_podcast_delete(podcast)
 
     def get_podcasts(self):
         def podcast_factory(dct, db):
