@@ -1430,7 +1430,7 @@ def gui_open(filename):
         elif gpodder.ui.osx:
             Popen(['open', filename], close_fds=True)
         else:
-            Popen(['xdg-open', filename], close_fds=True)
+            Popen(['gio', 'open', filename], close_fds=True)
         return True
     except:
         logger.error('Cannot open file/folder: "%s"', filename, exc_info=True)
