@@ -20,6 +20,7 @@
 #  Justin Forest <justin.forest@gmail.com> 2008-10-13
 #
 
+import io
 import json
 import logging
 import re
@@ -38,7 +39,7 @@ _ = gpodder.gettext
 
 
 # http://en.wikipedia.org/wiki/YouTube#Quality_and_formats
-# https://gist.github.com/Marco01809/34d47c65b1d28829bb17c24c04a0096f
+# https://github.com/ytdl-org/youtube-dl/blob/master/youtube_dl/extractor/youtube.py#L447
 
 # adaptive audio formats
 #   140  MP4   128k
@@ -129,6 +130,23 @@ formats = [
 ]
 formats_dict = dict(formats)
 
+# streaming formats and fallbacks to lower quality
+hls_144 = [91]
+hls_240 = [92] + hls_144
+hls_360 = [93] + hls_240
+hls_480 = [94] + hls_360
+hls_720 = [95] + hls_480
+hls_1080 = [96] + hls_720
+hls_formats = [
+    (96, (hls_1080, '9/1920x1080/9/0/115', 'MP4 1080p (1920x1080)')),   # N/A,       256 kbps
+    (95, (hls_720, '9/1280x720/9/0/115', 'MP4 720p (1280x720)')),       # N/A,       256 kbps
+    (94, (hls_480, '9/854x480/9/0/115', 'MP4 480p (854x480)')),         # N/A,       128 kbps
+    (93, (hls_360, '9/640x360/9/0/115', 'MP4 360p (640x360)')),         # N/A,       128 kbps
+    (92, (hls_240, '9/426x240/9/0/115', 'MP4 240p (426x240)')),         # N/A,        48 kbps
+    (91, (hls_144, '9/256x144/9/0/115', 'MP4 144p (256x144)')),         # N/A,        48 kbps
+]
+hls_formats_dict = dict(hls_formats)
+
 V3_API_ENDPOINT = 'https://www.googleapis.com/youtube/v3'
 CHANNEL_VIDEOS_XML = 'https://www.youtube.com/feeds/videos.xml'
 
@@ -137,27 +155,39 @@ class YouTubeError(Exception):
     pass
 
 
-def get_fmt_ids(youtube_config):
+def get_fmt_ids(youtube_config, allow_partial):
+    if allow_partial:
+        if youtube_config.preferred_hls_fmt_id == 0:
+            hls_fmt_ids = (youtube_config.preferred_hls_fmt_ids if youtube_config.preferred_hls_fmt_ids else [])
+        else:
+            format = hls_formats_dict.get(youtube_config.preferred_hls_fmt_id)
+            if format is None:
+                hls_fmt_ids = []
+            else:
+                hls_fmt_ids, path, description = format
+    else:
+        hls_fmt_ids = []
+
     if youtube_config.preferred_fmt_id == 0:
-        return (youtube_config.preferred_fmt_ids if youtube_config.preferred_fmt_ids else [])
+        return (youtube_config.preferred_fmt_ids + hls_fmt_ids if youtube_config.preferred_fmt_ids else hls_fmt_ids)
 
     format = formats_dict.get(youtube_config.preferred_fmt_id)
     if format is None:
-        return []
+        return hls_fmt_ids
     fmt_ids, path, description = format
-    return fmt_ids
+    return fmt_ids + hls_fmt_ids
 
 
 @registry.download_url.register
-def youtube_real_download_url(config, episode):
-    fmt_ids = get_fmt_ids(config.youtube) if config else None
-    res, duration = get_real_download_url(episode.url, fmt_ids)
+def youtube_real_download_url(config, episode, allow_partial):
+    fmt_ids = get_fmt_ids(config.youtube, allow_partial) if config else None
+    res, duration = get_real_download_url(episode.url, allow_partial, fmt_ids)
     if duration is not None:
         episode.total_time = int(int(duration) / 1000)
     return None if res == episode.url else res
 
 
-def get_real_download_url(url, preferred_fmt_ids=None):
+def get_real_download_url(url, allow_partial, preferred_fmt_ids=None):
     if not preferred_fmt_ids:
         preferred_fmt_ids, _, _ = formats_dict[22]  # MP4 720p
 
@@ -198,6 +228,25 @@ def get_real_download_url(url, preferred_fmt_ids=None):
                         and not playabilityStatus['liveStreamability'].get('liveStreamabilityRenderer', {}).get('displayEndscreen', False):
                     # playabilityStatus.liveStreamability -- video is or was a live stream
                     # playabilityStatus.liveStreamability.liveStreamabilityRenderer.displayEndscreen -- video has ended if present
+
+                    if allow_partial and 'streamingData' in player_response and 'hlsManifestUrl' in player_response['streamingData']:
+                        manifest = None
+                        url = player_response['streamingData']['hlsManifestUrl']
+                        while manifest is None:
+                            req = util.http_request(url, method='GET')
+                            if 'location' in req.msg:
+                                url = req.msg['location']
+                            else:
+                                manifest = req.read()
+                        manifest = manifest.decode().splitlines()
+
+                        urls = [line for line in manifest if line[0] != '#']
+                        itag_re = re.compile(r'/itag/([0-9]+)/')
+                        for url in urls:
+                            itag = itag_re.search(url).group(1)
+                            yield int(itag), [url, None]
+                        return
+
                     error_message = 'live stream'
                 elif 'streamingData' in player_response:
                     # DRM videos store url inside a cipher key - not supported
@@ -214,7 +263,7 @@ def get_real_download_url(url, preferred_fmt_ids=None):
             if error_message is not None:
                 raise YouTubeError('Cannot download video: %s' % error_message)
 
-            r4 = re.search('url_encoded_fmt_stream_map=([^&]+)', page)
+            r4 = re.search(r'url_encoded_fmt_stream_map=([^&]+)', page)
             if r4 is not None:
                 fmt_url_map = urllib.parse.unquote(r4.group(1))
                 for fmt_url_encoded in fmt_url_map.split(','):
@@ -224,7 +273,7 @@ def get_real_download_url(url, preferred_fmt_ids=None):
         fmt_id_url_map = sorted(find_urls(page), reverse=True)
 
         if not fmt_id_url_map:
-            drm = re.search('%22cipher%22%3A', page)
+            drm = re.search(r'%22(cipher|signatureCipher)%22%3A', page)
             if drm is not None:
                 raise YouTubeError('Unsupported DRM content found for video ID "%s"' % vid)
             raise YouTubeError('No formats found for video ID "%s"' % vid)
@@ -233,12 +282,12 @@ def get_real_download_url(url, preferred_fmt_ids=None):
         fmt_id_url_map = dict(fmt_id_url_map)
 
         for id in preferred_fmt_ids:
-            if re.search('\+', str(id)):
+            if re.search(r'\+', str(id)):
                 # skip formats that contain a + (136+140)
                 continue
             id = int(id)
             if id in formats_available:
-                format = formats_dict.get(id)
+                format = formats_dict.get(id) or hls_formats_dict.get(id)
                 if format is not None:
                     _, _, description = format
                 else:
@@ -255,15 +304,15 @@ def get_real_download_url(url, preferred_fmt_ids=None):
 
 
 def get_youtube_id(url):
-    r = re.compile('http[s]?://(?:[a-z]+\.)?youtube\.com/v/(.*)\.swf', re.IGNORECASE).match(url)
+    r = re.compile(r'http[s]?://(?:[a-z]+\.)?youtube\.com/v/(.*)\.swf', re.IGNORECASE).match(url)
     if r is not None:
         return r.group(1)
 
-    r = re.compile('http[s]?://(?:[a-z]+\.)?youtube\.com/watch\?v=([^&]*)', re.IGNORECASE).match(url)
+    r = re.compile(r'http[s]?://(?:[a-z]+\.)?youtube\.com/watch\?v=([^&]*)', re.IGNORECASE).match(url)
     if r is not None:
         return r.group(1)
 
-    r = re.compile('http[s]?://(?:[a-z]+\.)?youtube\.com/v/(.*)[?]', re.IGNORECASE).match(url)
+    r = re.compile(r'http[s]?://(?:[a-z]+\.)?youtube\.com/v/(.*)[?]', re.IGNORECASE).match(url)
     if r is not None:
         return r.group(1)
 
@@ -286,13 +335,13 @@ def for_each_feed_pattern(func, url, fallback_result):
     func() returns None, return fallback_result.
     """
     CHANNEL_MATCH_PATTERNS = [
-        'http[s]?://(?:[a-z]+\.)?youtube\.com/user/([a-z0-9]+)',
-        'http[s]?://(?:[a-z]+\.)?youtube\.com/profile?user=([a-z0-9]+)',
-        'http[s]?://(?:[a-z]+\.)?youtube\.com/channel/([-_a-zA-Z0-9]+)',
-        'http[s]?://(?:[a-z]+\.)?youtube\.com/rss/user/([a-z0-9]+)/videos\.rss',
-        'http[s]?://gdata.youtube.com/feeds/users/([^/]+)/uploads',
-        'http[s]?://gdata.youtube.com/feeds/base/users/([^/]+)/uploads',
-        'http[s]?://(?:[a-z]+\.)?youtube\.com/feeds/videos.xml\?channel_id=([-_a-zA-Z0-9]+)',
+        r'http[s]?://(?:[a-z]+\.)?youtube\.com/user/([a-z0-9]+)',
+        r'http[s]?://(?:[a-z]+\.)?youtube\.com/profile?user=([a-z0-9]+)',
+        r'http[s]?://(?:[a-z]+\.)?youtube\.com/rss/user/([a-z0-9]+)/videos\.rss',
+        r'http[s]?://(?:[a-z]+\.)?youtube\.com/channel/([-_a-z0-9]+)',
+        r'http[s]?://(?:[a-z]+\.)?youtube\.com/feeds/videos.xml\?channel_id=([-_a-z0-9]+)',
+        r'http[s]?://gdata.youtube.com/feeds/users/([^/]+)/uploads',
+        r'http[s]?://gdata.youtube.com/feeds/base/users/([^/]+)/uploads',
     ]
 
     for pattern in CHANNEL_MATCH_PATTERNS:
@@ -318,8 +367,8 @@ def get_channel_id_url(url):
     if 'youtube.com' in url:
         try:
             channel_url = ''
-            raw_xml_data = util.urlopen(url).read().decode('utf-8')
-            xml_data = xml.etree.ElementTree.fromstring(raw_xml_data)
+            raw_xml_data = io.BytesIO(util.urlopen(url).content)
+            xml_data = xml.etree.ElementTree.parse(raw_xml_data)
             channel_id = xml_data.find("{http://www.youtube.com/xml/schemas/2015}channelId").text
             channel_url = 'https://www.youtube.com/channel/{}'.format(channel_id)
             return channel_url
@@ -354,7 +403,7 @@ def get_cover(url):
 
         try:
             channel_url = get_channel_id_url(url)
-            html_data = util.urlopen(channel_url).read().decode('utf-8')
+            html_data = util.response_text(util.urlopen(channel_url))
             parser = YouTubeHTMLCoverParser()
             parser.feed(html_data)
             if parser.url:
@@ -385,7 +434,7 @@ def get_channel_desc(url):
 
         try:
             channel_url = get_channel_id_url(url)
-            html_data = util.urlopen(channel_url).read().decode('utf-8')
+            html_data = util.response_text(util.urlopen(channel_url))
             parser = YouTubeHTMLDesc()
             parser.feed(html_data)
             if parser.description:
@@ -437,7 +486,7 @@ def parse_youtube_url(url):
 
         if 'list=' in query:
             playlist_query = [query_value for query_value in query.split("&") if 'list=' in query_value][0]
-            playlist_id = playlist_query.strip("list=")
+            playlist_id = playlist_query[5:]
             query = 'playlist_id={playlist_id}'.format(playlist_id=playlist_id)
 
         path = '/feeds/videos.xml'
